package com.huimv.devicedata.service.impl;

import com.alibaba.fastjson.JSONObject;

import com.huimv.devicedata.entity.DeviceResponseEntity;
import com.huimv.devicedata.entity.DeviceUpdataEntity;
import com.huimv.devicedata.entity.SetTemperatureEntity;
import com.huimv.devicedata.entity.WarningInfoEntity;
import com.huimv.devicedata.repo.DeviceResponseEntityRepository;
import com.huimv.devicedata.repo.DeviceUpdataEntityRepository;
import com.huimv.devicedata.repo.SetTemperatureRepo;
import com.huimv.devicedata.repo.WarningInfoRepo;
import com.huimv.devicedata.service.IDeviceDataService;
import com.huimv.devicedata.utils.DateUtil;
import com.huimv.devicedata.utils.MathUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

/**
 * @Project : huimv.shiwan
 * @Package : com.huimv.biosafety.uface.controller
 * @Description : TODO
 * @Version : 1.0
 * @Author : ZhuoNing
 * @Create : 2020-12-25
 **/
@Service
public class DeviceDataServiceImpl implements IDeviceDataService {

    @Autowired
    private DeviceUpdataEntityRepository deviceUpdataEntityRepo;
    @Autowired
    private DateUtil dateUtil;
    @Autowired
    private MathUtil mathUtil;
    @Autowired
    private DeviceResponseEntityRepository deviceResponseEntityRepo;
    @Autowired
    private SetTemperatureRepo setTemperatureRepo;
    @Autowired
    private WarningInfoRepo WarningInfoRepo;

    @Override
    public void saveDeviceChangedUpData(JSONObject deviceUpDataJo) {
        String productId = deviceUpDataJo.getString("productId");
        String deviceId = deviceUpDataJo.getString("deviceId");
        long timestamp = deviceUpDataJo.getLong("timestamp");
        String upDataDate = dateUtil.formatLongDate(timestamp);
        String serviceId = deviceUpDataJo.getString("serviceId");
        String imsi = deviceUpDataJo.getString("IMSI");
        String imei = deviceUpDataJo.getString("IMEI");
        String deviceType = deviceUpDataJo.getString("deviceType");
        String messageType = deviceUpDataJo.getString("messageType");
        String protocol = deviceUpDataJo.getString("protocol");
        String tenantId = deviceUpDataJo.getString("tenantId");
        JSONObject businessDataJo = deviceUpDataJo.getJSONObject("payload");
        int dataVer = businessDataJo.getInteger("VER");
        int dataTime = businessDataJo.getInteger("TIME");
        int dataTemp = businessDataJo.getInteger("TEMP");
        int dataStc = businessDataJo.getInteger("STC");
        int dataRssi = businessDataJo.getInteger("RSSI");
        int dataHumi = businessDataJo.getInteger("HUMI");
        int dataBat = businessDataJo.getInteger("BAT");
        DeviceUpdataEntity deviceUpdataEntity = new DeviceUpdataEntity();
        deviceUpdataEntity.setProductId(productId);
        deviceUpdataEntity.setDeviceId(deviceId);
        deviceUpdataEntity.setTimestamp(upDataDate);
        deviceUpdataEntity.setServiceId(serviceId);
        deviceUpdataEntity.setImei(imei);
        deviceUpdataEntity.setImsi(imsi);
        deviceUpdataEntity.setDeviceType(deviceType);
        deviceUpdataEntity.setMessageType(messageType);
        deviceUpdataEntity.setProtocol(protocol);
        deviceUpdataEntity.setDataVer(dataVer);
        deviceUpdataEntity.setDataTime(dataTime);
        deviceUpdataEntity.setDataTemp(dataTemp);
        deviceUpdataEntity.setDataStc(dataStc);
        deviceUpdataEntity.setDataRssi(dataRssi);
        deviceUpdataEntity.setDataHumi(dataHumi);
        deviceUpdataEntity.setDataBat(dataBat);
        deviceUpdataEntity.setTenantId(tenantId);
        System.out.println("dataTemp: "+dataTemp);
        System.out.println("TempConvert: "+mathUtil.formatDecimalNumber(dataTemp*0.1,1));
        deviceUpdataEntity.setDataTempConvert(mathUtil.formatDecimalNumber(dataTemp*0.1,1));
        deviceUpdataEntity.setDataHumiConvert(mathUtil.formatDecimalNumber(dataHumi*0.1,1));
        deviceUpdataEntity.setDataBatConvert(mathUtil.formatDecimalNumber(dataBat*0.01,1));
//        System.out.println("dataTemp/10:"+mathUtil.formatDecimalNumber(dataTemp*0.1,1));
//        System.out.println("dataHumi/10:"+mathUtil.formatDecimalNumber(dataHumi*0.1,1));
//        System.out.println("dataBat/10:"+mathUtil.formatDecimalNumber(dataBat*0.01,1));
        //
        deviceUpdataEntityRepo.saveAndFlush(deviceUpdataEntity);
    }

    /**
     * 设备命令响应
     *
     * @param deviceResponseData
     */
    @Override
    public void saveDeviceResponseData(JSONObject deviceResponseData) {

        //json中取出数据
        String tenantId = deviceResponseData.getString("tenantId");
        String productId = deviceResponseData.getString("productId");
        String messageType = deviceResponseData.getString("messageType");
        String deviceId = deviceResponseData.getString("deviceId");
        Integer taskId = deviceResponseData.getInteger("taskId");
        JSONObject result = deviceResponseData.getJSONObject("result");
        String resultCode = result.getString("resultCode");
        JSONObject resultDetail = result.getJSONObject("resultDetail");
        Integer length = resultDetail.getInteger("Length");
        String rspdata = resultDetail.getString("Rspdata");

        //封装到实体类
        DeviceResponseEntity deviceResponseEntity = new DeviceResponseEntity();
        deviceResponseEntity.setTenanId(tenantId);
        deviceResponseEntity.setProductId(productId);
        deviceResponseEntity.setMessageType(messageType);
        deviceResponseEntity.setDeviceId(deviceId);
        deviceResponseEntity.setTaskId(taskId);
        deviceResponseEntity.setResultCode(resultCode);
        deviceResponseEntity.setResultDetailLength(length);
        deviceResponseEntity.setResultDetailRspData(rspdata);
        //保存到数据库
        deviceResponseEntityRepo.saveAndFlush(deviceResponseEntity);
    }

    @Override
    public void saveWarningInfo(JSONObject deviceUpDataJo) {
        String productId = deviceUpDataJo.getString("productId");
        String deviceId = deviceUpDataJo.getString("deviceId");
        long timestamp = deviceUpDataJo.getLong("timestamp");
        String upDataDate = dateUtil.formatLongDate(timestamp);
        String serviceId = deviceUpDataJo.getString("serviceId");
        String imsi = deviceUpDataJo.getString("IMSI");
        String imei = deviceUpDataJo.getString("IMEI");
        String deviceType = deviceUpDataJo.getString("deviceType");
        String messageType = deviceUpDataJo.getString("messageType");
        String protocol = deviceUpDataJo.getString("protocol");
        String tenantId = deviceUpDataJo.getString("tenantId");
        JSONObject businessDataJo = deviceUpDataJo.getJSONObject("payload");
        int dataVer = businessDataJo.getInteger("VER");
        int dataTime = businessDataJo.getInteger("TIME");
        int dataTemp = businessDataJo.getInteger("TEMP");
        int dataStc = businessDataJo.getInteger("STC");
        int dataRssi = businessDataJo.getInteger("RSSI");
        int dataHumi = businessDataJo.getInteger("HUMI");
        int dataBat = businessDataJo.getInteger("BAT");
        Double tempVal = mathUtil.formatDecimalNumber(dataTemp*0.1,1);

        // 读取温度参数阈值
        SetTemperatureEntity setTemperatureEntity = setTemperatureRepo.getOne(2);
        String high = setTemperatureEntity.getHighTemperature();
        String low = setTemperatureEntity.getLowTemperature();
        Double highVal = Double.parseDouble(high);
        Double lowVal = Double.parseDouble(low);
        if(tempVal>highVal){
            // 生成高温报警
            WarningInfoEntity warningInfoEntity = new WarningInfoEntity();
            warningInfoEntity.setLQID(deviceId);
            warningInfoEntity.setType("出现高温报警:"+tempVal);
            warningInfoEntity.setUploadTime(dateUtil.getTimestamp());
            warningInfoEntity.setWarningType(2);
            warningInfoEntity.setShowStatus(true);
            WarningInfoRepo.saveAndFlush(warningInfoEntity);
        }
        if(tempVal<lowVal){
            // 生成低端报警
            WarningInfoEntity warningInfoEntity = new WarningInfoEntity();
            warningInfoEntity.setLQID(deviceId);
            warningInfoEntity.setType("出现低温报警:"+tempVal);
            warningInfoEntity.setUploadTime(dateUtil.getTimestamp());
            warningInfoEntity.setWarningType(2);
            warningInfoEntity.setShowStatus(true);
            WarningInfoRepo.saveAndFlush(warningInfoEntity);
        }
        // 湿度数据
        Double humiVal = mathUtil.formatDecimalNumber(dataHumi*0.1,1);
        // 读取湿度参数阈值
        SetTemperatureEntity setHumiEntity = setTemperatureRepo.getOne(3);
        String highHumi = setHumiEntity.getHighTemperature();
        String lowHumi = setHumiEntity.getLowTemperature();
        Double highHumiVal = Double.parseDouble(highHumi);
        Double lowHumiVal = Double.parseDouble(lowHumi);
        if(humiVal>highHumiVal){
            // 生成过高湿度报警
            WarningInfoEntity warningInfoEntity = new WarningInfoEntity();
            warningInfoEntity.setLQID(deviceId);
            warningInfoEntity.setType("出现过高湿度报警:"+humiVal);
            warningInfoEntity.setUploadTime(dateUtil.getTimestamp());
            warningInfoEntity.setWarningType(2);
            warningInfoEntity.setShowStatus(true);
            WarningInfoRepo.saveAndFlush(warningInfoEntity);
        }
        if(humiVal<lowHumiVal){
            // 生成过低湿度报警
            WarningInfoEntity warningInfoEntity = new WarningInfoEntity();
            warningInfoEntity.setLQID(deviceId);
            warningInfoEntity.setType("出现过低湿度报警:"+humiVal);
            warningInfoEntity.setUploadTime(dateUtil.getTimestamp());
            warningInfoEntity.setWarningType(2);
            warningInfoEntity.setShowStatus(true);
            WarningInfoRepo.saveAndFlush(warningInfoEntity);
        }
    }
}

